/*-------------------------------------------------------------------------*\
 |                                                                         |
 |                                                                         |
 |  WE_EXT.C - A Sample DLL Extension Processor for WinEdit                |
 |                                                                         |
 |                                                                         |
\*-------------------------------------------------------------------------*/
#define STRICT
#define _WINDLL
#include <windows.h>
#include "we_ext.h"
#include "private.h"

#define NOREF(a)  {a=a;}

HMENU hTrackMenu;
BOOL bWait=TRUE;
BOOL bCapture=TRUE;
char szCommand[256];

/*-------------------------------------------------------------------------*\
 |                                                                         |
 |  Function:   WE_ExtensionProc                                           |
 |                                                                         |
 |  Purpose:    WinEdit calls this function with the WEN_* messages and    |
 |              whenever a user-defined menu item or accelerator is        |
 |              accessed.                                                  |
 |                                                                         |
 |  Parameters: HWND hWnd   - WinEdit's window handle                      |
 |                                                                         |
 |              UINT wParam - Message ID.  If wParam is >= WE_EXTFIRST,    |
 |                            the DLL is being requested to perform the    |
 |                            user-defined action.                         |
 |                                                                         |
\*-------------------------------------------------------------------------*/
UINT FAR PASCAL WE_ExtensionProc(HWND hWnd,     /* WinEdit's window handle */
                                 HANDLE hInst,  /* instance identifier     */
                                 UINT wParam,   /* command ID              */
                                 LONG lParam)   /* additional information  */
   {

   switch (wParam)
      {

      case WEN_LOADMENU:

         /*  This is the menu WinEdit will display when there
          *  is at least one document window open.  Return NULL
          *  to use the default WinEdit menu.
          *
          */

         return (UINT)LoadMenu(hInst, "MyMenu");
         break;

      case WEN_LOADSHORTMENU:

         /*  this is the menu WinEdit will display when there
             *  are no document windows open.  Return NULL
             *  to use the default WinEdit menu.
             *
             */
   
            return (UINT)LoadMenu(hInst, "MyShortMenu");
            return NULL;
            break;
   
      case WEN_LOADACCELS:

         /*  To re-define the WinEdit command keys, load your
          *  own accelerator table here.  Return NULL to
          *  use the default WinEdit accelerators.
          *
          */

         return (UINT)LoadAccelerators (hInst,"MyAccels");
         break;

      case WEN_GETWINDOWMENU:

         /*  WinEdit needs the handle of the submenu to
          *  append MDI document names to.  The hWnd parameter
          *  is used to send the handle to the main menu.
          *  This message will not be sent if you return
          *  NULL to the WEN_LOADMENU message.
          */
         return (UINT)GetSubMenu ((HMENU)hWnd, WINDOWMENU);
         break;

      case WEN_GETMACROMENU:

         /*  WinEdit needs the handle of the submenu to
          *  append macro names to.  The hWnd parameter
          *  is used to send the handle to the main menu.
          *  This message will not be sent if you return
          *  NULL to the WEN_LOADMENU message.
          */
         return (UINT)GetSubMenu ((HMENU)hWnd, MACROMENU);
         break;
         
      case WEN_END:

         /*  WinEdit is shutting down.  Do any clean-up processing
          *  here.
          */
         if (hTrackMenu)
            {
            DestroyMenu(hTrackMenu);
            hTrackMenu = (HMENU)NULL;
            }
         return TRUE;
         break;

      case WEN_INITMENU:

         /*  This message is sent before showing any drop down
          *  menu items.  Respond by setting any checkmarks,
          *  graying any inapplicable items, etc.
          *
          */
         return InitMenu(hWnd);
         break;

      case WEN_RBUTTONDOWN:

         /*  WinEdit sends right button clicks to the
          *  extension processor.  lParam contains the
          *  x and y coordinates of the mouse cursor,
          *  in screen coordinates.
          *
          *  This sample duplicates WinEdit's default action
          *  of putting up a TrackPopupMenu.  You can
          *  modify the menu to include the commands you
          *  use most often.
          */
         if (!hTrackMenu)
            {

            hTrackMenu = CreatePopupMenu();
            if (!hTrackMenu)
               break;
               
            AppendMenu(hTrackMenu,MF_STRING,IDM_FILEOPEN,"&Open...\tF3");
            AppendMenu(hTrackMenu,MF_STRING,IDM_FILESAVE,"&Save\tF2");                 
            AppendMenu(hTrackMenu,MF_STRING,IDM_FILEPRINT,"&Print\tF9");                
            AppendMenu(hTrackMenu,MF_SEPARATOR,0,0L);

            AppendMenu(hTrackMenu,MF_STRING,IDM_EDITUNDO,"&Undo\tAlt+BkSp");           
            AppendMenu(hTrackMenu,MF_STRING,IDM_EDITREDO,"&Redo\tCtrl+BkSp");          
            AppendMenu(hTrackMenu,MF_SEPARATOR,0,0L);

            AppendMenu(hTrackMenu,MF_STRING,IDM_EDITCUT,"Cu&t\tShift+Del");           
            AppendMenu(hTrackMenu,MF_STRING,IDM_EDITCOPY,"&Copy\tCtrl+Ins");           
            AppendMenu(hTrackMenu,MF_STRING,IDM_EDITPASTE,"&Paste\tShift+Ins");         
            AppendMenu(hTrackMenu,MF_SEPARATOR,0,0L);

            AppendMenu(hTrackMenu,MF_STRING,IDM_SEARCHFIND,"&Find...\tF5");              
            AppendMenu(hTrackMenu,MF_STRING,IDM_SEARCHNEXT,"&Repeat Last Find\tCtrl+F5");
            AppendMenu(hTrackMenu,MF_STRING,IDM_SEARCHCHANGE,"&Change...\tF6");            
            AppendMenu(hTrackMenu,MF_SEPARATOR,0,0L);

            AppendMenu(hTrackMenu,MF_STRING,IDM_SEARCHNEXTERR,"&Next Error\tShift+F3");    
            AppendMenu(hTrackMenu,MF_STRING,IDM_SEARCHPREVERR,"&Previous Error\tShift+F4");
            AppendMenu(hTrackMenu,MF_SEPARATOR,0,0L);

            AppendMenu(hTrackMenu,MF_STRING,IDM_HELPKEYWORDS,"&Key Word Help\tShift+F1");  
            }
            
         if (hTrackMenu)
            {
            POINT pt;

            pt = MAKEPOINT(lParam);
            TrackPopupMenu(hTrackMenu,0,pt.x-10,pt.y-6,0,hWnd,0L);
            }
            
         return TRUE;   
         break;


      case WEN_RBUTTONDOWNS:
         {

#if 1
         /*  This is an example of trapping the right
          *  mouse button.  The #else code is a duplicate
          *  of WinEdit's default action.
          *
          */

         char szTemp[64];

         MessageBeep(0);
         edEditGetCurrentWord(hWnd,szTemp,63);
         if (szTemp[0])
            WinHelp(hWnd,"WIN31WH.HLP",HELP_KEY,(DWORD)(LPSTR)szTemp);

         return TRUE;
#else         
         edHelpKeyWord(hWnd);
#endif
         return TRUE;
         }



      /*  You can define your own commands in the range
       *  WE_EXTFIRST to WE_EXTLAST that can be attached to
       *  menu items or accelerators.
       */

      case EXT_EXAMPLE:
         {

         /* do a grep on the current word */

         char szWord[64];
         edEditGetCurrentWord(hWnd,szWord,63);
         if (szWord[0])
            wsprintf(szCommand,"tee.com fgrep.com -M %s *.c",(LPSTR)szWord);
         if (DialogBox(hInst,"CommandBox",hWnd,CommandDlgProc))
            edRunCommand(hWnd, bWait, bCapture, szCommand);
         return TRUE;
         break;
         }

      case EXT_IF:
         {
         /* Ctrl+I:  'C' template for   if (  )

          */
         UINT wColNo;

         edEditInsertString(hWnd,"if (  )");

         wColNo  = edGetColumnNumber(hWnd);

         edEditGoToColumn(hWnd,wColNo-2);

         return TRUE;
         break;
         }

      case EXT_FOR:
         {
         /* Ctrl+F:  'C' template for   for ( ; ; )

          */
         UINT wColNo;

         edEditInsertString(hWnd,"for ( ; ; )");

         wColNo  = edGetColumnNumber(hWnd);

         edEditGoToColumn(hWnd,wColNo-5);

         return TRUE;
         break;
         }

      case EXT_SWITCH:
         {
         /* Ctrl+S:  'C' template for   switch (  )
                                           {

                                           }
          */
         UINT wColNo;
         UINT wLineNo;

         edEditInsertString(hWnd,"switch (  )\r   {\r\r}");

         wColNo  = edGetColumnNumber(hWnd);
         wLineNo = edGetLineNumber(hWnd);

         edEditGoToColumn(hWnd,wColNo+5);

         edEditGoToLine(hWnd,wLineNo-3);

         return TRUE;
         break;
         }

      default:

         /* return NULL to all messages not processed. */

         break;

      }  /* end switch (wParam) */

   return NULL;

   }



BOOL FAR PASCAL LibMain(HANDLE hInstance, UINT wDataSeg, UINT cbHeap, LPSTR lpszCmdLine)
   {
   return TRUE;

   NOREF(hInstance);
   NOREF(wDataSeg);
   NOREF(cbHeap);
   NOREF(lpszCmdLine);
   }


int FAR PASCAL WEP(int nParameter)
   {
   return TRUE;

   NOREF(nParameter);
   }


BOOL FAR PASCAL CommandDlgProc(HWND hDlg, UINT msg, UINT wParam, LONG lParam)
   {
   switch (msg)
      {
      case WM_INITDIALOG:
         CheckDlgButton(hDlg,IDD_WAIT,bWait);
         CheckDlgButton(hDlg,IDD_CAPTURE,bCapture);
         SetDlgItemText(hDlg,IDD_COMMAND,szCommand);

      case WM_COMMAND:
         switch(wParam)
            {
            case IDOK:
               bWait = IsDlgButtonChecked(hDlg,IDD_WAIT);
               bCapture = IsDlgButtonChecked(hDlg,IDD_CAPTURE);
               if (GetDlgItemText(hDlg,IDD_COMMAND,szCommand,sizeof(szCommand)))
                  {
                  EndDialog(hDlg,TRUE);
                  break;
                  }
               /* else fall through */

            case IDCANCEL:
               EndDialog(hDlg,FALSE);
               break;

            default:
               return (FALSE);
            }
         break;

      default:
         return(FALSE);
      }
   return (TRUE);

   NOREF(lParam);
   }



int InitMenu(HWND hWnd)
   {
   UINT wStatus;
   HMENU hCurrentMenu;
   POINT ptStart,ptEnd;

   hCurrentMenu = GetMenu(hWnd);

   /*  if there is a current selection, enable the cut & copy
    *  commands.
    */
   wStatus = (UINT)edGetSelectionState(hWnd, &ptStart, &ptEnd);
   if (!wStatus)
      wStatus = MF_GRAYED;
   else
      wStatus = MF_ENABLED;
   EnableMenuItem(hCurrentMenu, IDM_EDITCUT,  wStatus);
   EnableMenuItem(hCurrentMenu, IDM_EDITCOPY, wStatus);

   /*  if there is text on the clipboard, enable the paste
    *  command.
    */
   if (OpenClipboard(hWnd))
      {
      if (IsClipboardFormatAvailable(CF_TEXT)
            || IsClipboardFormatAvailable(CF_OEMTEXT))
         EnableMenuItem(hCurrentMenu, IDM_EDITPASTE, MF_ENABLED);
      else
         EnableMenuItem(hCurrentMenu, IDM_EDITPASTE, MF_GRAYED);
      CloseClipboard();
      }
   else
      EnableMenuItem(hCurrentMenu, IDM_EDITPASTE, MF_GRAYED);

   /* set the Undo, Redo, Insert, and WordWrap menu items */
   wStatus = (UINT)edGetUndoState(hWnd);
   if (!wStatus)
      wStatus = MF_GRAYED;
   else
      wStatus = MF_ENABLED;
   EnableMenuItem(hCurrentMenu, IDM_EDITUNDO, wStatus);

   wStatus = (UINT)edGetRedoState(hWnd);
   if (!wStatus)
      wStatus = MF_GRAYED;
   else
      wStatus = MF_ENABLED;
   EnableMenuItem(hCurrentMenu, IDM_EDITREDO, wStatus);

   wStatus = (UINT)edGetWordWrapState(hWnd);
   if (!wStatus)
      wStatus = MF_UNCHECKED;
   else
      wStatus = MF_CHECKED;
   CheckMenuItem (hCurrentMenu, IDM_EDITTOGGLEWRAP, MF_BYCOMMAND|wStatus);

   wStatus = (UINT)edGetInsertState(hWnd);
   if (!wStatus)
      wStatus = MF_UNCHECKED;
   else
      wStatus = MF_CHECKED;
   CheckMenuItem (hCurrentMenu, IDM_EDITTOGGLEINS, MF_BYCOMMAND|wStatus);

   return TRUE;   /* we handled it, don't return 0 */
   }
